@@ -23,14 +23,18 @@ try:
     from custom_exceptions.salesforce_exceptions import *
     from utilities.df_utils import *
     from utilities.async_utils import *
+    from utilities.sf_query_processors import *
     from decorators import *
 except:
     from salesforce.custom_exceptions.salesforce_exceptions import *
     from salesforce.utilities.df_utils import *
     from salesforce.utilities.async_utils import *
+    from salesforce.utilities.sf_query_processors import *
     from salesforce.decorators import *
 import asyncio
 from functools import wraps, partial
+from pandas import json_normalize
+
 
 
 
@@ -605,6 +609,36 @@ class Sf:
         else:
             raise ValueError('include_name_compound_fields must be a boolean.')
         return compound_field_names
+    
+    
+    async def get_picklist_values_by_object_record_type(sf, object: str):
+        object_record_type_ids = sf.query(f"Select Id, Name From RecordType Where SobjectType = '{object}'").apply(lambda row: {"Id": row['Id'], "Name": row['Name']}, axis=1).to_list()
+        object_describe = sf.sf_api_call(f"/services/data/{sf.api_version}/ui-api/object-info/{object}")['fields']
+        controlling_fields = []
+        for field, values in object_describe.items():
+            controlling_fields.append({"Field": field, "Controlling Field": "" if values['controllingFields'] == [] else "; ".join(values['controllingFields'])})
+        controlling_fields = pd.DataFrame(controlling_fields)
+
+        def record_type_picklist_retriever(sf, object, record_type_id, record_type_name):
+            result = []
+            for key, value in sf.sf_api_call(f"/services/data/{sf.api_version}/ui-api/object-info/{object}/picklist-values/{record_type_id}")['picklistFieldValues'].items():
+                result.append({"Field_API": key, "controllerValues": value['controllerValues'], "defaultValue":  value['defaultValue'], "values": value['values']})
+            final_result_pd = json_normalize(result, ['values'], ['defaultValue', 'controllerValues', 'Field_API'])
+            ## Converts the validFor numerical codes i.e. [0,2] to it's corresponding text values by looking it up on the controllerValues column i.e. {'Web': 0, 'Phone Inquiry': 1, 'Partner Reference': 2 ... }
+            ## [0,2] -> ['Web', 'Partner Reference']
+            final_result_pd['validFor'] = final_result_pd.apply(lambda row: "\n".join([{v: k for k, v in row['controllerValues'].items()}[value] for value in row['validFor']]), axis=1)
+            final_result_pd.drop(columns=['controllerValues'], inplace=True)
+            final_result_pd[['defaultValueLabel', 'defaultValueValue']] = json_normalize(final_result_pd['defaultValue'])[['label','value']]
+            final_result_pd.drop(columns=['defaultValue'], inplace=True)
+            final_result_pd = pd.merge(final_result_pd, controlling_fields, left_on='Field_API', right_on='Field', how='left').drop(columns=['Field']).copy()
+            final_result_pd['RecordTypeName'] = record_type_name
+            final_result_pd['Object'] = object
+            return final_result_pd
+
+        async_picklist_rt_retrieve = async_wrap(record_type_picklist_retriever)
+        results = pd.concat(await asyncio.gather(*[async_picklist_rt_retrieve(sf, object, record_type_id['Id'], record_type_id['Name']) for record_type_id in object_record_type_ids]))
+        results = results[['Object', 'RecordTypeName', 'Field_API', 'Controlling Field', 'defaultValueLabel', 'defaultValueValue', 'label', 'value', 'validFor']]
+        return results
 
     ### ----------------------------------------------------------------------------------------------------
     ### Synchronous Metadata Functions
